@@ -81,10 +81,12 @@ class GrandIsoExecutor(NetworkXExecutor):
             only_positive_edges_motif,
             self.graph,
             is_node_attr_match=_node_attr_match_fn,
+            is_edge_attr_match=lambda _1, _2, _3, _4: True,
         )
 
         _edge_constraint_validator = (
-            self._validate_edge_constraints if not self._host_is_multigraph
+            self._validate_edge_constraints
+            if not self._host_is_multigraph
             else (
                 self._validate_multigraph_all_edge_constraints
                 if self._multigraph_edge_match == "all"
@@ -95,9 +97,7 @@ class GrandIsoExecutor(NetworkXExecutor):
         results = []
         for r in graph_matches:
             if _doesnt_have_any_of_motifs_negative_edges(r) and (
-                _edge_constraint_validator(
-                    r, self.graph, motif.list_edge_constraints()
-                )
+                _edge_constraint_validator(r, self.graph, motif.list_edge_constraints())
                 # and self._validate_node_constraints(
                 #     r, self.graph, motif.list_node_constraints()
                 # )
