@@ -1,5 +1,5 @@
 """
-Copyright 2020 The Johns Hopkins University Applied Physics Laboratory.
+Copyright 2021 The Johns Hopkins University Applied Physics Laboratory.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -174,7 +174,7 @@ class Neo4jExecutor(Executor):
                 raise ValueError(f"Could not export graph: {e}")
 
             self._tamarind_provisioner = tamarind.Neo4jDockerProvisioner(
-                autoremove_container=self._autoremove_container,
+                autoremove_containers=self._autoremove_container,
                 max_memory_size=self._max_memory_size,
                 initial_heap_size=self._initial_heap_size,
             )
@@ -182,7 +182,7 @@ class Neo4jExecutor(Executor):
 
         elif import_directory:
             self._tamarind_provisioner = tamarind.Neo4jDockerProvisioner(
-                autoremove_container=self._autoremove_container,
+                autoremove_containers=self._autoremove_container,
                 max_memory_size=self._max_memory_size,
                 initial_heap_size=self._initial_heap_size,
             )
@@ -212,6 +212,13 @@ class Neo4jExecutor(Executor):
 
     def _create_container(self, import_dir: str):
         # Create a docker container:
+
+        _run_before = (
+            f"""./bin/neo4j-admin import --id-type STRING --nodes={self._entity_labels['node']}="""
+            + f""""/import/export-neurons-.*.csv" --relationships={self._entity_labels['edge']['DEFAULT']}="""
+            + """"/import/export-synapses-.*.csv" """
+        )
+
         self._tamarind_container_id = str(uuid4())
         (
             self._running_container,
@@ -219,8 +226,9 @@ class Neo4jExecutor(Executor):
         ) = self._tamarind_provisioner.start(
             self._tamarind_container_id,
             import_path=f"{os.getcwd()}/{import_dir}",
-            run_before=f"""./bin/neo4j-admin import --id-type STRING --nodes:{self._entity_labels['node']} "/import/export-neurons-.*.csv" --relationships:{self._entity_labels['edge']['DEFAULT']} "/import/export-synapses-.*.csv" """,
+            run_before=_run_before,
             wait=self._wait_for_boot,
+            wait_attempt_limit=self.max_retries,
         )
         self._created_container = True
         container_is_ready = False
@@ -236,12 +244,25 @@ class Neo4jExecutor(Executor):
                         f"Could not connect to neo4j container {self._running_container}. "
                         "For more information, see https://github.com/aplbrain/dotmotif/wiki/Troubleshooting-Neo4jExecutor."
                     )
-                time.sleep(5)
+                time.sleep(3)
         self.G = self._tamarind_provisioner[self._tamarind_container_id]
 
     def _teardown_container(self):
         self._tamarind_provisioner.stop(self._tamarind_container_id)
 
+    def create_index(self, attribute_name: str):
+        """
+        Create a new index on the given node attribute.
+
+        Note that edge attributes are NOT supported.
+        """
+        self.run(
+            f"""
+        CREATE INDEX index_name IF NOT EXISTS FOR (n:{self._entity_labels["node"]})
+        ON (n.{attribute_name})
+        """
+        )
+
     def run(self, cypher: str, cursor=True):
         """
         Run an arbitrary cypher command.
@@ -259,12 +280,12 @@ class Neo4jExecutor(Executor):
             return self.G.run(cypher).to_table()
         return self.G.run(cypher)
 
-    def count(self, motif: "dotmotif", limit=None) -> int:
+    def count(self, motif: "dotmotif.Motif", limit=None) -> int:
         """
         Count a motif in a larger graph.
 
         Arguments:
-            motif (dotmotif.dotmotif)
+            motif (dotmotif.Motif)
 
         """
         qry = self.motif_to_cypher(
@@ -274,12 +295,12 @@ class Neo4jExecutor(Executor):
             qry += f" LIMIT {limit}"
         return int(self.G.run(qry).to_ndarray())
 
-    def find(self, motif: "dotmotif", limit=None, cursor=True):
+    def find(self, motif: "dotmotif.Motif", limit=None, cursor=True):
         """
         Find a motif in a larger graph.
 
         Arguments:
-            motif (dotmotif.dotmotif)
+            motif (dotmotif.Motif)
 
         """
         qry = self.motif_to_cypher(motif, static_entity_labels=self._entity_labels)
@@ -291,7 +312,7 @@ class Neo4jExecutor(Executor):
 
     @staticmethod
     def motif_to_cypher(
-        motif: "dotmotif", count_only: bool = False, static_entity_labels: dict = None,
+        motif: "dotmotif.Motif", count_only: bool = False, static_entity_labels: dict = None,
     ) -> str:
         """
         Output a query suitable for Cypher-compatible engines (e.g. Neo4j).
